;******************************************************************************
;    Filename: Bale Stroke Counter                                            *
;    Date:     May. 29, 2004                                                  *
;    File Version: 2.0                                                        *
;                                                                             *
;    Author:   Kent Kenison                                                   *
;    Company:  Kenison Creations                                              *
;******************************************************************************
	LIST P=16F872, N=400				;directive to define processor
	#include <P16F872.INC>				;processor specific variable definitions
;******************************************************************************
	__CONFIG _CP_OFF & _HS_OSC & _PWRTE_ON  & _WDT_OFF & _LVP_ON & _BODEN_OFF & _DEBUG_OFF & _CPD_OFF
;******************************************************************************
;EEPROM data
; Data to be programmed into the Data EEPROM is defined here

		ORG	0x2100

		DE	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

;******************************************************************************

;Interface Lines
	; PORTA
#DEFINE DIGIT_5			PORTA,5			; RA5, DIGIT_5
#DEFINE DIGIT_4			PORTA,4			; RA4, DIGIT_4
#DEFINE DIGIT_3			PORTA,3			; RA3, DIGIT_3
#DEFINE DIGIT_2			PORTA,2			; RA2, DIGIT_2
#DEFINE DIGIT_1			PORTA,1			; RA1, DIGIT_1
#DEFINE DIGIT_0			PORTA,0			; RA0, DIGIT_0

	; PORTB
#DEFINE DOG_LED			PORTB,7			; RB7, 
#DEFINE TIE_LED			PORTB,6			; RB6, 
;#DEFINE RB5			PORTB,5			; RB5, 
#DEFINE TIE_SENSOR		PORTB,4			; RB4, Tie Sensor
;#DEFINE RB3			PORTB,3			; RB3, PGM Pin, Output Only
#DEFINE COUNT_RST		PORTB2,2		; RB2, Counter Reset
#DEFINE STROKE_RST		PORTB2,1		; RB1, Stroke Reset
#DEFINE DOG_SENSOR		PORTB,0			; RB0, Dog Sensor

	; PORTC
;#DEFINE LED_DATA_7		PORTC,7			; RC7,
#DEFINE LED_DATA_6		PORTC,6			; RC6, SEG_G
#DEFINE LED_DATA_5		PORTC,5			; RC5, SEG_F
#DEFINE LED_DATA_4		PORTC,4			; RC4, SEG_E
#DEFINE LED_DATA_3		PORTC,3			; RC3, SEG_D
#DEFINE LED_DATA_2		PORTC,2			; RC2, SEG_C
#DEFINE LED_DATA_1		PORTC,1			; RC1, SEG_B
#DEFINE LED_DATA_0		PORTC,0			; RC0, SEG_A

		CBLOCK	0x022
		STROKE_COUNT_1
		STROKE_COUNT_2
		STROKE_LED_1
		STROKE_LED_2
		TIE_TRIGGER
		DOG_TRIGGER
		TOTAL_1
		TOTAL_2
		TOTAL_3
		TOTAL_4
		DELAY_TRIGGER
		PORTB_HOLDER
		PORTB2
		DELAY_2
		ENDC

;******************************************************************************
	; Reset vector
	ORG		0x000					; RESET vector location
    GOTO	START

	ORG		0x004					; Interrupt vector location
INTERRUPT	
    CLRF	STATUS					; Bank 0
    BTFSC	PIR1, TMR2IF			; Timer 2 overflowed?
	GOTO	T2_OVRFL        		; YES, Service the Timer2
	BTFSC	INTCON,INTF
	GOTO	INT_TRIGGER				; Dog Sensor?
	BTFSC	INTCON,RBIF
	GOTO	PORTB_TRIGGER			; Tie Sensor?
	BCF		INTCON,T0IF
	MOVLW	0x0C0
	MOVWF	TMR0
	CALL	REDRAW_DISPLAY
	RETFIE

START							;Begin Program
	CLRF	STATUS		            	; Do initialization, Select bank 0
	CLRF	PCLATH		            	; Keep in lower 2KByte
	CLRF	PORTA		            	; ALL PORT output should output Low. 
	CLRF	PORTB	                  	; ALL PORT output should output Low.       
	CLRF	PORTC                   	; ALL PORT output should output Low.
	CLRF	T1CON						; Reset Timer 1
	CLRF	INTCON		            	; Clear int-flags, Disable interrupts
	CLRF	PIR1						; Clear peripheral Interrupts
	BSF		STATUS,RP0	            	; Select bank 1
	CLRF	PIE1^0x080					; Disable peripheral Interrupts
	MOVLW	0x006                    	; Configure all pins
	MOVWF	ADCON1^0x080				;   as digital Inputs
	CLRF	TRISA^0x080					; RA5-0 Outputs
    MOVLW	B'00010111'	            	; RB1-0 Inputs, RB7-2 Outputs
    MOVWF	TRISB^0x080					; 
    MOVLW	0x000
    MOVWF	OPTION_REG^0x080
;	BCF		OPTION_REG^0x080,7			; Enable pull-ups on RB0-RB1
;	BCF		OPTION_REG^0x080,6			; Interupt Edge Select Bit, Falling
	CLRF	TRISC^0x080     	       	; RC7-6 Inputs, RC5-0 Outputs
	CLRF	STATUS		            	; Select bank 0
	MOVLW	0x0A0
	MOVWF	TMR0
    CLRF	STROKE_COUNT_1
    CLRF	STROKE_COUNT_2
    CLRF	STROKE_LED_1
    CLRF	STROKE_LED_2
    CLRF	TOTAL_1
    CLRF	TOTAL_2
    CLRF	TOTAL_3
    CLRF	TOTAL_4
	CALL	READ_EEPROM				; Reload Data
; Set Up Sensor Ports Interrupts
    MOVLW	0x0F8
    MOVWF	INTCON
;	BSF		INTCON,INTE				; Enable PortB0 Interrupt
;	BSF		INTCON,RBIE				; Enable PortB Interrupt
;	BSF		INTCON,PEIE				; Enable Peripheral Interrupts
;	BSF		INTCON,GIE				; Enable All Interrupts
	BCF		DOG_TRIGGER,0
	BCF		TIE_TRIGGER,0
;******************************************************************************	
MAIN_LOOP							; Where it goes when it is running    
	CLRF	STATUS
    BCF		DOG_LED
    BCF		TIE_LED
	BTFSC	DOG_TRIGGER,0
	CALL	INC_STROKE_COUNT       		; INC Stroke
	BTFSC	TIE_TRIGGER,0
	CALL	INC_COUNT_N_RESET			; Inc Count
	CALL	CHECK_BTNS
	GOTO	MAIN_LOOP

;******************************************************************************	
INC_STROKE_COUNT
	INCF	STROKE_COUNT_1,F          		; Increment the 1 digit
	MOVLW   .10                 			; A decimal 10
	SUBWF   STROKE_COUNT_1,W    	      	;
	BTFSS   STATUS,Z           				; Did this subtract cause Zero Result?
	GOTO	$+3
	INCF    STROKE_COUNT_2,F          		; Increment the 10 digit
	CLRF	STROKE_COUNT_1
	CALL	SAVE_EEPROM
	BCF		DOG_TRIGGER,0
	RETURN

INC_COUNT_N_RESET
	INCF	TOTAL_1,F          		; Increment the 1 digit
	MOVLW   .10                 	; A decimal 10
	SUBWF   TOTAL_1,W    	      	;
	BTFSS   STATUS,Z           		; Did this subtract cause Zero Result?
	GOTO	N_RESET
	INCF    TOTAL_2,F          		; Increment the 10 digit
	CLRF	TOTAL_1
	MOVLW   .10                 	; A decimal 10
	SUBWF   TOTAL_2,W    	      	;
	BTFSS   STATUS,Z           		; Did this subtract cause Zero Result?
	GOTO	N_RESET
	INCF    TOTAL_3,F          		; Increment the 100 digit
	CLRF	TOTAL_2
	MOVLW   .10                 	; A decimal 10
	SUBWF   TOTAL_3,W    	      	;
	BTFSS   STATUS,Z           		; Did this subtract cause Zero Result?
	GOTO	N_RESET
	INCF    TOTAL_4,F          		; Increment the 1000 digit
	CLRF	TOTAL_3
N_RESET
	MOVF	STROKE_COUNT_1,W
	MOVWF	STROKE_LED_1
	MOVF	STROKE_COUNT_2,W
	MOVWF	STROKE_LED_2
	CLRF	STROKE_COUNT_1
	CLRF	STROKE_COUNT_2
	CALL	SAVE_EEPROM
	BCF		TIE_TRIGGER,0
	RETURN
	
;****************************************************************************	
CHECK_BTNS
	MOVF	PORTB,W
	MOVWF	PORTB_HOLDER
	MOVLW	0x005
	MOVWF	DELAY_2
	BCF		DELAY_TRIGGER,1
	CALL	TMR_2_ON
BTN_LOOP							; Where it goes when it is Delayed    
	BTFSS	DELAY_TRIGGER,1
	GOTO	BTN_LOOP
	BCF		DELAY_TRIGGER,1
	DECFSZ	DELAY_2,F					; 
	GOTO	BTN_LOOP				
    CALL	TMR_2_OFF
	MOVF	PORTB_HOLDER,W
	ANDWF	PORTB,W
	MOVWF	PORTB2
	BTFSS	STROKE_RST        			; RESET?
	CALL	RESET_STROKE       			; RESET Stroke
	BTFSS	COUNT_RST        			; RESET?
	CALL	RESET_COUNTER				; RESET Counter
	RETURN

RESET_STROKE
	CLRF	STROKE_COUNT_1
	CLRF	STROKE_COUNT_2
	CLRF	STROKE_LED_1
	CLRF	STROKE_LED_2
	CALL	SAVE_EEPROM
	RETURN

RESET_COUNTER
	CLRF	TOTAL_1
	CLRF	TOTAL_2
	CLRF	TOTAL_3
	CLRF	TOTAL_4
	CALL	SAVE_EEPROM
	RETURN
;****************************************************************************	
REDRAW_DISPLAY
	CLRF	PORTC
	MOVF	STROKE_LED_1,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_0
	NOP
	NOP
	BCF		DIGIT_0
	MOVF	STROKE_LED_2,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_1
	NOP
	NOP
	BCF		DIGIT_1
	MOVF	TOTAL_1,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_2
	NOP
	NOP
	BCF		DIGIT_2
	MOVF	TOTAL_2,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_3
	NOP
	NOP
	BCF		DIGIT_3
	MOVF	TOTAL_3,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_4
	NOP
	NOP
	BCF		DIGIT_4

	MOVF	TOTAL_4,W
	CALL	NUM_TABLE
	MOVWF	PORTC
	BSF		DIGIT_5
	NOP
	NOP
	BCF		DIGIT_5
	RETURN

NUM_TABLE
	ADDWF   PCL,F           	; Determine Offset into table
	RETLW   B'00111111'			; ASCII value of "0" in W register
	RETLW   B'00000110'			; ASCII value of "1" in W register
	RETLW   B'01011011'			; ASCII value of "2" in W register
	RETLW   B'01001111'			; ASCII value of "3" in W register
	RETLW   B'01100110'			; ASCII value of "4" in W register
	RETLW   B'01101101'			; ASCII value of "5" in W register
	RETLW   B'01111101'			; ASCII value of "6" in W register
	RETLW   B'00000111'			; ASCII value of "7" in W register
	RETLW   B'01111111'			; ASCII value of "8" in W register
	RETLW   B'01101111'			; ASCII value of "9" in W register
NUM_TABLE_END
;****************************************************************************	
INT_TRIGGER
	BCF		INTCON,INTF
    BSF		DOG_LED
	BSF		DOG_TRIGGER,0
	RETFIE

PORTB_TRIGGER
	BCF		INTCON,RBIF
PORTB_TRIGGER_LOOP
	BTFSC	TIE_SENSOR
	RETFIE				
    BSF		TIE_LED
	BSF		TIE_TRIGGER,0
	RETFIE
	
T2_OVRFL
	BCF		STATUS,RP0           	; Select Bank 0
    BCF		PIR1, TMR2IF			; Clear timer 2 interrupt flag/STEP
	BSF		DELAY_TRIGGER,1
	BCF		STATUS,RP0           	; Select Bank 0
	RETFIE

TMR_2_OFF
	BCF		T2CON,TMR2ON			; Turn Timer1 off
	BSF		STATUS,RP0           	; Select Bank 1
	BCF		PIE1^0x080,TMR2IE		; Disable TMR1 interrupt
	BCF		STATUS,RP0           	; Select Bank 0
	RETURN
	
TMR_2_ON
	CLRF	T2CON
	BSF		STATUS,RP0           	; Select Bank 1
	MOVLW	0x040
	MOVWF 	PR2^0x080 				; Load PWM period.
	BSF 	PIE1^0x080,TMR2IE 		; Set TMR2=PR2 interrupt mask bit.
	BCF		STATUS,RP0           	; Select Bank 0
	MOVLW 	0x07F					; Enable timer and 
	MOVWF 	T2CON 					; Load prescale value 16:1 (b100+b010)
	BSF 	INTCON,PEIE 			; Set peripheral interrupt mask bit.
	BSF 	INTCON,GIE 				; Enable interrupts.
	RETURN

;****************************************************************************	
READ_EEPROM
    BSF		STATUS,RP1	            	; 
    BCF		STATUS,RP0	            	; Select bank 2
 	CLRF	EEADR^0x0100				; Start Loc
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	STROKE_COUNT_1				; From EEPROM

	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	STROKE_COUNT_2				; From EEPROM

	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	STROKE_LED_1					; From EEPROM

	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	STROKE_LED_2					; From EEPROM

	INCF	EEADR^0x0100,F			    ; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	TOTAL_1						; From EEPROM

	INCF	EEADR^0x0100,F			    ; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	TOTAL_2						; From EEPROM

	INCF	EEADR^0x0100,F			    ; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	TOTAL_3						; From EEPROM

	INCF	EEADR^0x0100,F			    ; Point to next Loc in EEPROM
    BSF		STATUS,RP0	            	; Select bank 3
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,RD
    BCF		STATUS,RP0	            	; Select bank 2
	MOVF	EEDATA^0x0100,W
	MOVWF	TOTAL_4						; From EEPROM
	CLRF	STATUS
	RETURN

SAVE_EEPROM
    BSF		STATUS,RP1	            	; 
    BCF		STATUS,RP0	            	; Select bank 2
 	CLRF	EEADR^0x0100				; Start Loc
	MOVF	STROKE_COUNT_1,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	STROKE_COUNT_2,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	STROKE_LED_1,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	STROKE_LED_2,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	TOTAL_1,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM	

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	TOTAL_2,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	TOTAL_3,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM

    BCF		STATUS,RP0	            	; Select bank 2
	INCF	EEADR^0x0100,F				; Point to next Loc in EEPROM
	MOVF	TOTAL_4,W
	MOVWF	EEDATA^0x0100				; Load EEPROM
	CALL	SEND_TO_EEPROM
	CLRF	STATUS
	RETURN

SEND_TO_EEPROM
    BSF		STATUS,RP0	            	; Bank 3
    BSF		STATUS,RP1	            	; 
	BCF		EECON1^0x0180,EEPGD
	BSF		EECON1^0x0180,WREN
	BCF		INTCON,GIE					; 
	MOVLW	0x055
	MOVWF	EECON2^0x0180
	MOVLW	0x0AA
	MOVWF	EECON2^0x0180
	BSF		EECON1^0x0180,WR
	BTFSC	EECON1^0x0180,WR			; Done?
	GOTO	$-1							;-1 LINE
	BCF		EECON1^0x0180,WREN
	BSF		INTCON,GIE					; 
	RETURN
	
;****************************************************************************		
		END
